package io.xjar.boot;

import java.io.Closeable;
import java.util.zip.Checksum;
import java.util.zip.CheckedOutputStream;
import java.util.zip.CRC32;
import java.io.ByteArrayOutputStream;
import java.io.ByteArrayInputStream;
import io.xjar.XKit;
import org.apache.commons.compress.archivers.zip.ZipArchiveEntry;
import java.util.jar.Attributes;
import java.util.jar.Manifest;
import org.apache.commons.compress.archivers.ArchiveEntry;
import io.xjar.jar.XJarDecryptor;
import io.xjar.XUnclosedOutputStream;
import io.xjar.XUnclosedInputStream;
import org.apache.commons.compress.archivers.jar.JarArchiveOutputStream;
import org.apache.commons.compress.archivers.jar.JarArchiveInputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.InputStream;
import java.io.FileOutputStream;
import java.io.FileInputStream;
import java.io.File;
import io.xjar.key.XKey;
import io.xjar.XEntryFilter;
import io.xjar.jar.XJarAllEntryFilter;
import io.xjar.XConstants;
import io.xjar.XDecryptor;
import org.apache.commons.compress.archivers.jar.JarArchiveEntry;
import io.xjar.XEntryDecryptor;

public class XBootDecryptor extends XEntryDecryptor<JarArchiveEntry> implements XDecryptor, XConstants
{
    private final int level;
    
    public XBootDecryptor(final XDecryptor xDecryptor) {
        this(xDecryptor, (XEntryFilter<JarArchiveEntry>)new XJarAllEntryFilter());
    }
    
    public XBootDecryptor(final XDecryptor xDecryptor, final XEntryFilter<JarArchiveEntry> filter) {
        this(xDecryptor, -1, filter);
    }
    
    public XBootDecryptor(final XDecryptor xDecryptor, final int level) {
        this(xDecryptor, level, (XEntryFilter<JarArchiveEntry>)new XJarAllEntryFilter());
    }
    
    public XBootDecryptor(final XDecryptor xDecryptor, final int level, final XEntryFilter<JarArchiveEntry> filter) {
        super(xDecryptor, (XEntryFilter)filter);
        this.level = level;
    }
    
    public void decrypt(final XKey key, final File src, final File dest) throws IOException {
        try (FileInputStream fis = new FileInputStream(src);
             FileOutputStream fos = new FileOutputStream(dest)) {
            this.decrypt(key, fis, fos);
        }
    }
    
    public void decrypt(final XKey key, final InputStream in, final OutputStream out) throws IOException {
        JarArchiveInputStream zis = null;
        JarArchiveOutputStream zos = null;
        try {
            zis = new JarArchiveInputStream(in);
            zos = new JarArchiveOutputStream(out);
            zos.setLevel(this.level);
            final XUnclosedInputStream nis = new XUnclosedInputStream((InputStream)zis);
            final XUnclosedOutputStream nos = new XUnclosedOutputStream((OutputStream)zos);
            final XJarDecryptor xJarDecryptor = new XJarDecryptor(this.xDecryptor, this.level, this.filter);
            JarArchiveEntry entry;
            while ((entry = zis.getNextJarEntry()) != null) {
                if (!entry.getName().startsWith(XBootDecryptor.XJAR_SRC_DIR) && !entry.getName().endsWith("XJAR-INF/")) {
                    if (entry.getName().endsWith("XJAR-INF/INDEXES.IDX")) {
                        continue;
                    }
                    if (entry.isDirectory()) {
                        final JarArchiveEntry jarArchiveEntry = new JarArchiveEntry(entry.getName());
                        jarArchiveEntry.setTime(entry.getTime());
                        zos.putArchiveEntry((ArchiveEntry)jarArchiveEntry);
                    }
                    else if (entry.getName().equals("META-INF/MANIFEST.MF")) {
                        final Manifest manifest = new Manifest((InputStream)nis);
                        final Attributes attributes = manifest.getMainAttributes();
                        final String mainClass = attributes.getValue("Boot-Main-Class");
                        if (mainClass != null) {
                            attributes.putValue("Main-Class", mainClass);
                            attributes.remove(new Attributes.Name("Boot-Main-Class"));
                        }
                        final JarArchiveEntry jarArchiveEntry2 = new JarArchiveEntry(entry.getName());
                        jarArchiveEntry2.setTime(entry.getTime());
                        zos.putArchiveEntry((ArchiveEntry)jarArchiveEntry2);
                        manifest.write((OutputStream)nos);
                    }
                    else if (entry.getName().startsWith("BOOT-INF/classes/")) {
                        final JarArchiveEntry jarArchiveEntry = new JarArchiveEntry(entry.getName());
                        jarArchiveEntry.setTime(entry.getTime());
                        zos.putArchiveEntry((ArchiveEntry)jarArchiveEntry);
                        final XBootJarArchiveEntry xBootJarArchiveEntry = new XBootJarArchiveEntry((ZipArchiveEntry)entry);
                        final boolean filtered = this.filtrate((Object)xBootJarArchiveEntry);
                        final XDecryptor decryptor = (XDecryptor)(filtered ? this.xDecryptor : this.xNopDecryptor);
                        try (OutputStream eos = decryptor.decrypt(key, (OutputStream)nos)) {
                            XKit.transfer((InputStream)nis, eos);
                        }
                    }
                    else if (entry.getName().startsWith("BOOT-INF/lib/")) {
                        final byte[] data = XKit.read((InputStream)nis);
                        final ByteArrayInputStream lib = new ByteArrayInputStream(data);
                        final boolean need = xJarDecryptor.predicate((InputStream)lib);
                        lib.reset();
                        if (need) {
                            final ByteArrayOutputStream bos = new ByteArrayOutputStream();
                            final CheckedOutputStream cos = new CheckedOutputStream(bos, new CRC32());
                            xJarDecryptor.decrypt(key, (InputStream)lib, (OutputStream)cos);
                            final JarArchiveEntry jarArchiveEntry3 = new JarArchiveEntry(entry.getName());
                            jarArchiveEntry3.setMethod(0);
                            jarArchiveEntry3.setSize((long)bos.size());
                            jarArchiveEntry3.setTime(entry.getTime());
                            jarArchiveEntry3.setCrc(cos.getChecksum().getValue());
                            zos.putArchiveEntry((ArchiveEntry)jarArchiveEntry3);
                            final ByteArrayInputStream bis = new ByteArrayInputStream(bos.toByteArray());
                            XKit.transfer((InputStream)bis, (OutputStream)nos);
                        }
                        else {
                            final JarArchiveEntry jarArchiveEntry2 = new JarArchiveEntry(entry.getName());
                            jarArchiveEntry2.setMethod(0);
                            jarArchiveEntry2.setSize(entry.getSize());
                            jarArchiveEntry2.setTime(entry.getTime());
                            jarArchiveEntry2.setCrc(entry.getCrc());
                            zos.putArchiveEntry((ArchiveEntry)jarArchiveEntry2);
                            XKit.transfer((InputStream)lib, (OutputStream)nos);
                        }
                    }
                    else {
                        final JarArchiveEntry jarArchiveEntry = new JarArchiveEntry(entry.getName());
                        jarArchiveEntry.setTime(entry.getTime());
                        zos.putArchiveEntry((ArchiveEntry)jarArchiveEntry);
                        XKit.transfer((InputStream)nis, (OutputStream)nos);
                    }
                    zos.closeArchiveEntry();
                }
            }
            zos.finish();
        }
        finally {
            XKit.close((Closeable)zis);
            XKit.close((Closeable)zos);
        }
    }
}
